/**
 *
 * @author yutent<yutent.io@gmail.com>
 * @date 2021/05/30 11:43:54
 */

export class EventEmitter {
  //
  __events__ = Object.create(null)

  $on(name, fn) {
    if (this.__events__[name]) {
      this.__events__[name].push(fn)
    } else {
      this.__events__[name] = [fn]
    }
  }

  $once(name, fn) {
    function fn1(...args) {
      fn(...args)
    }
    if (this.__events__[name]) {
      this.__events__[name].push(fn)
    } else {
      this.__events__[name] = [fn]
    }
  }

  $off(name, fn) {
    if (this.__events__[name]) {
      if (fn) {
        this.__events__[name] = this.__events__[name].filter(it => it !== fn)
      } else {
        this.__events__[name] = []
      }
    }
  }

  $emit(name, ...args) {
    if (this.__events__[name]) {
      for (let fn of this.__events__[name]) {
        try {
          fn.apply(this, args)
        } catch (e) {
          console.error(e)
        }
      }
    }
  }

  $destroy() {
    this.__events__ = Object.create(null)
  }
}
